%macro fdataphC1(dsn,outcomes,surv0,surv1,weight,byvars,mainpred,covars,clvars,outdata);
/*
This macro summarizes multiple Cox regression models in a PROC REPORT format.  
It allows for multiple outcomes, multiple BY variables, ONE PRIMARY PREDICTOR,
additional covariates, and staggered entry, with a start and stop survival time.

Output from this macro includes forest plots.  For situations with multiple
stratification output, such as evaluating multiple potential effect modifiers, use %FDATAPH2.

For this macro, the main predictor [MAINPRED] must be numeric and must be in the CLASS statement.

Parameters
    dsn         Input data set name
    outcomes    Outcome variables. List with spaces -- must be in the same order
                as survival times!!!
    surv0       Survival start times for each outcome -- must be in the same order as outcomes!!!
                Use this for models with both start and stop times for survival.  If all start times are
                zero, just leave this blank
    surv1       Survival stop times. If SURV0 is blank, this is a standard survival event time.
    weight      Weight variable.  Especially for inverse probability weights. ignored if blank.
    byvars      BY variables.  The order determines the order in the figure/table
    mainpred    Main categorical predictor.  This will be included in the figure/table.
                MUST BE NUMERIC AND IN THE CLASS STATEMENT [CLSVAR].
    covars      Covariates.  These are not included in the figure/table
    clvars      The CLASS statement for categorical variables.  This is the actual CLASS statement
                [except for the initial word 'class'].  See examples.
    outdata     Output data set for Reports. May include libname.

*/
/* Save current ODS destination */
data _null_; set sashelp.vdest;
call symputx("olddest",style);
run;

/* make working data set */
data repwork; set &dsn;
keep &outcomes &surv0 &surv1 &weight &byvars &mainpred &covars;
run;


/* count outcomes, create outcome and survival time variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&outcomes,&count);
  %if &riskf ne  %then %do;
    %let out&count = &riskf; /* outcome variables */
    %let surva&count = %qscan(&surv0,&count); /* survival start times */
    %let survb&count = %qscan(&surv1,&count); /* survival start times */
    %let outcount = &count;
    %let count = %eval(&count+1); /* number of outcomes */
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;

/* Count BY variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&byvars,&count);
  %if &riskf ne  %then %do;
    %let byvar&count = &riskf;
    %let bycount = &count;
    %let count = %eval(&count+1);
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;
%if &byvars ne %then %do;
proc sort data=repwork; by &byvars;
run;
%end;

/* Run Cox Models */
%do outnum = 1 %to &outcount;
ods output ParameterEstimates=pe&outnum;
proc phreg data=repwork;
%if &byvars ne %then %do;
by &byvars;
%end;
class &clvars;
%if &surv0 eq  %then %do;
model &&survb&outnum * &&out&outnum(0) = &mainpred &covars /rl;
%end;
%else %do;
model (&&surva&outnum,&&survb&outnum) * &&out&outnum(0) = &mainpred &covars /rl;
%end;
%if &weight ne %then %do;
  weight &weight;
%end;
run;

data pe&outnum;set pe&outnum;
Parameter = upcase(parameter);
if Parameter eq upcase("&mainpred");
class2 = classval0/1;
rename class2=Classval0;
drop Classval0;
run;
proc contents data=pe&outnum;
run;



proc contents data=repwork noprint out=conmainx;
run;
data conmainx; set conmainx;
  if upcase(name) eq upcase("&mainpred") then do;
     call symputx("mainlab",Label);
     call symputx("mainfor",format);
     end;
  run;

/* Obtain main risk group counts, person time */

proc contents data=pe&outnum noprint out=conoutx;
run;
  data conoutx; set conoutx;
  if name eq 'Parameter' then call symputx("plength",Length);
  run;

/* Create person-time variables;*/
data trepwork; set repwork;
%if &&surva&outnum ne  %then %do;
ptime = &&survb&outnum - &&surva&outnum ;
%end;
%else %do;
ptime = &&survb&outnum;
%end;
run;


proc means data=trepwork noprint nway;
class &byvars &mainpred;
var ptime;
output out=ptimes mean=meantime median=medtime;
run;

data ptimes; set ptimes;
length Parameter $ &plength;
Parameter = upcase("&mainpred");
Classval0=&mainpred; 
keep &byvars Parameter Classval0 meantime medtime;
run;

ods output crosstabfreqs=ctfc;
proc freq data=repwork;
%if &byvars ne %then %do;
by &byvars;
%end;
tables &mainpred*&&out&outnum;
run;
proc sort data=ctfc;
by &byvars &mainpred &&out&outnum;
data ct_all; set ctfc; by &byvars &mainpred &&out&outnum;
retain nsubs npcts;
if (&&out&outnum eq .) and (&mainpred ne .) then do;
  nsubs = frequency;
  npcts = put(percent,4.1);
end;
if (&&out&outnum eq 1) and (&mainpred ne .) then do;
  dsubs = frequency;
  dpcts = put(rowpercent,4.1);
end;
if last.&mainpred;
length subp $ 30;
length subd $ 30;
length EV_subs $ 30;
length Parameter $ &plength;
subp = cat(nsubs,' (', npcts,')');
subd = cat(dsubs,' (', dpcts,')');
if &mainpred ne .;
Parameter = upcase("&mainpred");
Classval0 = &mainpred;
EV_subs = cat(dsubs,'/',nsubs);
keep &byvars &mainpred subp subd nsubs npcts dsubs dpcts EV_subs Parameter Classval0 ;
run;



proc sort data=ptimes; by &byvars Parameter Classval0;
proc sort data=ct_all; by &byvars Parameter Classval0;
proc sort data=pe&outnum; by &byvars Parameter Classval0;


data pe&outnum; merge pe&outnum ct_all ptimes; by &byvars Parameter Classval0;
outcome = "&&out&outnum";
if (upcase("&mainpred")) eq upcase(parameter);
Classval0 = &mainpred;
run;
%end;

/* Concatenate output from all outcomes */
data PE_all; set 
%do outnum = 1 %to &outcount;
 pe&outnum
 %end;
 ;
 length PECI $ 30;
 PE = put(HazardRatio,best4.);
 PEU = put(HRUpperCL,best4.);
 PEL = put(HRLowerCL,best4.);
 if HazardRatio ne . then  PECI = cat(PE,' (', PEL,',', PEU,')');
 else PECI='REF';
 drop PE PEU PEL;
 rename HazardRatio=PE HRUpperCL=PEU HRLowerCL=PEL;
 run;



/* Combine variable names and categories */
 proc sort data=PE_all; by outcome &byvars Parameter;
 data PE_all; set PE_all; by outcome &byvars Parameter;
/* Alternating color bands by BY variable */
   retain stgrp;
   if _n_ eq 1 then stgrp = 0;
%if &byvars ne %then %do;
   if first.&&byvar&bycount then stgrp = stgrp+1;
%end;
%else %do;
   stgrp = stgrp+1;
%end;
   scolor = mod(stgrp,2);

 length betaname $ 35;
 length betaname2 $ 15;
 parmtext = put(Parameter,parmf.);
 betaname2 = put(classval0,predf.);
 betaname = cat(trim(parmtext),' ',betaname2);

 label  PECI='HR (95% CI)' outcome='Outcome' meantime='Mean Time' medtime='Median Time'
 probchisq='P-value' betaname='Risk Group' betaname2="&mainlab" EV_subs='Events/N';
 format parameter parmf. outcome outf. medtime meantime 6.1;
 run;
data &outdata; set PE_all;
length macname $ 20;
%if &outcount ne 1 %then %do;
multout = 1;
%end;
macname='fdataphC1'; 
run;
%mend fdataphC1;
run;


%macro fdataphC2(dsn,outcomes,surv0,surv1,weight,stratvars,mainpred,covars,clvars,outdata);
/*
This macro summarizes multiple Cox regression models in a PROC REPORT format.  
It allows for multiple outcomes, one or more main predictors, multiple stratifying variables
additional covariates, and staggered entry, with a start and stop survival time.

Output from this macro can be used to create forest plots.  For situations without multiple
stratification variables, but with multiple BY variables, use %FDATAPH1.

For this macro, the main predictor [MAINPRED] must be numeric and must be in the CLASS statement.

Parameters
    dsn         Input data set name
    outcomes    Outcome variables. List with spaces -- must be in the same order
                as survival times!!!
    surv0       Survival start times for each outcome -- must be in the same order as outcomes!!!
                Use this for models with both start and stop times for survival.  If all start times are
                zero, just leave this blank
    surv1       Survival stop times. If SURV0 is blank, this is a standard survival event time.
    weight      Weight variable.  Especially for inverse probability weights. ignored if blank.
    stratvars   Stratifying variables.  The order determines the order in the figure/table.
                These variables will appear in the same column. Required!
    mainpred    Main variables.  These will be included in the figure/table
                MUST BE NUMERIC AND IN THE CLASS STATEMENT [CLSVARS]
    covars      Covariates.  These are not included in the figure/table
    clvars      The CLASS statement for categorical variables.  This is the actual CLASS statement
                [except for the initial word 'class'].  See examples below.
    outdata     Output data set for Reports. May include libname.

*/
/* Save current ODS destination */
data _null_; set sashelp.vdest;
call symputx("olddest",style);
run;

/* make working data set */
data repwork; set &dsn;
keep &outcomes &surv0 &surv1 &weight &stratvars &mainpred &covars;
run;

/* count outcomes, create outcome and survival time variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&outcomes,&count);
  %if &riskf ne  %then %do;
    %let out&count = &riskf; /* outcome variables */
    %let surva&count = %qscan(&surv0,&count); /* survival start times */
    %let survb&count = %qscan(&surv1,&count); /* survival stop times */
    %let outcount = &count;
    %let count = %eval(&count+1); /* number of outcomes */
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;

/* Count STRATUM variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&stratvars,&count);
  %if &riskf ne  %then %do;
    %let stratvar&count = &riskf;
    %let stratcount = &count;
    %let count = %eval(&count+1);
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;


/* Run Cox Models */
%do outnum = 1 %to &outcount;
  %do stratnum = 1 %to &stratcount;
  proc sort data=repwork; by &&stratvar&stratnum;
  run;
  ods output ParameterEstimates=pe&outnum._&stratnum;
  proc phreg data=repwork;
  by &&stratvar&stratnum;
  class &clvars;
  %if &surv0 eq  %then %do;
    model &&survb&outnum * &&out&outnum(0) = &mainpred &covars /rl;
  %end;
  %else %do;
    model (&&surva&outnum,&&survb&outnum) * &&out&outnum(0) = &mainpred &covars /rl;
  %end;
  %if &weight ne %then %do;
  weight &weight;
  %end;
  run;

data pe&outnum._&stratnum;set pe&outnum._&stratnum;
Parameter = upcase(parameter);
if Parameter eq upcase("&mainpred");
class2 = classval0/1;
rename class2=Classval0;
drop Classval0;
run;

proc contents data=repwork noprint out=conmainx;
run;
data conmainx; set conmainx;
  if upcase(name) eq upcase("&mainpred") then do;
     call symputx("mainlab",Label);
     call symputx("mainfor",format);
     end;
  run;


proc contents data=pe&outnum._&stratnum noprint out=conoutx;
  run;

data conoutx; set conoutx;
  length label2 $ 20;
  if name eq 'Parameter' then call symputx("plength",Length);
  if name =: 'Class' then call symputx("clength",Length);
  if upcase(name)=upcase("&&stratvar&stratnum") then do;
  label2 = substr(label,1,20);
  call symputx("labval",Label2);
  call symputx("forval",format);
  call symputx("slength",Length);
  end;
  run;

 

/* Create person-time variables;*/
data trepwork; set repwork;
%if &&surva&outnum ne  %then %do;
ptime = &&survb&outnum - &&surva&outnum ;
%end;
%else %do;
ptime = &&survb&outnum;
%end;
run;


proc means data=trepwork noprint nway;
class &&stratvar&stratnum &mainpred;
var ptime;
output out=ptimes mean=meantime median=medtime;
run;

data ptimes; set ptimes;
length Parameter $ &plength;
stratum1 = "&&stratvar&stratnum";
Parameter = upcase("&mainpred");
Classval0 = &mainpred;
keep stratum1 Parameter Classval0 meantime medtime;
run;


ods output crosstabfreqs=ctfc;
proc freq data=repwork;
by &&stratvar&stratnum;
tables &mainpred*&&out&outnum;
run;
proc sort data=ctfc;
by &&stratvar&stratnum &mainpred &&out&outnum;

data ct_all; set ctfc; by &&stratvar&stratnum &mainpred &&out&outnum;
retain nsubs npcts;
if (&&out&outnum eq .) and (&mainpred ne .) then do;
  nsubs = frequency;
  npcts = put(percent,4.1);
end;
if (&&out&outnum eq 1) and (&mainpred ne .) then do;
  dsubs = frequency;
  dpcts = put(rowpercent,4.1);
end;
if last.&mainpred;
length subp $ 30;
length subd $ 30;
length EV_subs $ 30;
length Parameter $ &plength;
length stratum1 $ &slength;
subp = cat(nsubs,' (', npcts,')');
subd = cat(dsubs,' (', dpcts,')');
if &mainpred ne .;
stratum1 = "&&stratvar&stratnum";
Parameter = upcase("&mainpred");
Classval0 = &mainpred;
EV_subs = cat(dsubs,'/',nsubs);
keep &&stratvar&stratnum stratum1 &mainpred subp subd nsubs npcts dsubs dpcts EV_subs Parameter Classval0 ;
run;
  
  data pe&outnum._&stratnum; set pe&outnum._&stratnum;
  length stratum1 $ &slength;
  stratum1 = "&&stratvar&stratnum";
  Parameter = upcase(Parameter);
  if Parameter = upcase("&mainpred");
run;

  proc sort data=ptimes; by stratum1 Parameter Classval0;
  proc sort data=pe&outnum._&stratnum; by stratum1 Parameter Classval0;
  proc sort data=ct_all; by stratum1 Parameter Classval0;

data pe&outnum._&stratnum; merge pe&outnum._&stratnum ct_all ptimes; by stratum1 Parameter Classval0;
  length strattext $ 15;
    length stratum $ 30;
    outcome = "&&out&outnum";
  strattext = put(&&stratvar&stratnum,&forval..);
  stratum = cat("&labval",': ',strattext);
  run;
  %end;
%end;

/* Concatenate output from all outcomes */
data PE_all; set 
%do outnum = 1 %to &outcount;
  %do stratnum = 1 %to &stratcount;
     pe&outnum._&stratnum
  %end;
%end;
 ;
 length PECI $ 30;
 PE = put(HazardRatio,best4.);
 PEU = put(HRUpperCL,best4.);
 PEL = put(HRLowerCL,best4.);
 if HazardRatio ne . then  PECI = cat(PE,' (', PEL,',', PEU,')');
 else PECI='REF';
 drop PE PEU PEL;
 rename HazardRatio=PE HRUpperCL=PEU HRLowerCL=PEL;
 label PE='HR (95% CI)';
 run;


proc sort data=PE_all; by outcome stratum Parameter;;
 data PE_all; set PE_all; by outcome stratum Parameter;
/* Alternating color bands by stratum */
   retain stgrp;
   if _n_ eq 1 then stgrp = 0;
   if first.stratum then stgrp = stgrp+1;
   scolor = mod(stgrp,2);
 length betaname $ 35;
 length betaname2 $ 15;
 parmtext = put(Parameter,parmf.);
 betaname2 = put(classval0,predf.);
 betaname = cat(trim(parmtext),' ',betaname2);

 label  PECI='HR (95% CI)' outcome='Outcome' stratum='Stratum' meantime='Mean Time'
probchisq='P-value' betaname="&mainlab" betaname2="&mainlab" medtime='Median Time' EV_subs='Events/N';
 format outcome outf. meantime 6.1;
 drop stgrp;
 run;

proc sort data=pe_all; by outcome stratum &mainpred;
run;
data &outdata; set PE_all;
length macname $ 20;
%if &outcount ne 1 %then %do;
multout = 1;
%end;
macname='fdataphC2'; 
run;
%mend fdataphc2;
run;

%macro fdatagmC1(dsn,outcomes,dist,link,offset,weight,byvars,mainpred,covars,clvars,outdata,bout=NO,expyn=YES);
/*
This macro summarizes multiple Cox regression models in a PROC REPORT format.  
It allows for multiple outcomes, multiple BY variables, one or more main predictors,
additional covariates, and staggered entry, with a start and stop survival time.

For this macro, the main predictor [MAINPRED] must be numeric and must be in the CLASS statement.

Output from this macro can be used to create forest plots.  For situations with multiple
stratification output, such as evaluating multiple potential effect modifiers, use %FDATAPH2.

Parameters
    dsn         Input data set name
    outcomes    Outcome variables. List with spaces.
    dist        Distribution. Must be one of "NORMAL", "BIN", "POISSON" or "NB".
    link        Link function
    offset      Offsets for Poisson/negative binomial models.  Can be left blank. Must be paired
                with the outcome variables [one offset for each outcome].  Should be the log
                value of person-time, etc.
    weight      Weight variable.  Especially for inverse probability weights. ignored if blank.
    byvars      BY variables.  The order determines the order in the figure/table
    mainpred    A single, categorical main predictor.  This will be included in the figure/table
                MUST BE NUMERIC AND IN THE CLASS STATEMENT [CLSVARS]
    covars      Covariates.  These are not included in the figure/table
    clvars      The CLASS statement for categorical variables.  This is the actual CLASS statement
                [except for the initial word 'class'].  See examples below.
    outdata     Output data set for Reports and Forest plots. May include libname.
    bout        Is this a binary outcome?  Default is NO.  If this is a Poisson/NB model and has a
                dichotomous outcome, set to YES to get counts of events.
    expyn       Exponentiate estimates?  YES=yes,NO=no. Default is YES.


*/
%if (&dist eq BIN) or (&bout ne NO) %then %do;
 %let bout = YES;
%end;

/* Save current ODS destination */
data _null_; set sashelp.vdest;
call symputx("olddest",style);
run;

/* make working data set */
data repwork; set &dsn;
keep &outcomes &offset &weight &byvars &mainpred &covars;
run;

%let dist = %upcase(&dist);

/* count outcomes, create outcome variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&outcomes,&count);
  %if &riskf ne  %then %do;
    %let out&count = &riskf; /* outcome variables */
    %let outcount = &count;
    %let count = %eval(&count+1); /* number of outcomes */
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;

/* create offset variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&offset,&count);
  %if &riskf ne  %then %do;
    %let off&count = &riskf; /* offset variables */
    %let count = %eval(&count+1); /* number of outcomes */
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;



/* Count BY variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&byvars,&count);
  %if &riskf ne  %then %do;
    %let byvar&count = &riskf;
    %let bycount = &count;
    %let count = %eval(&count+1);
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;
%if &byvars ne %then %do;
proc sort data=repwork; by &byvars;
run;
%end;

/* Run genmod Models */
%do outnum = 1 %to &outcount;
ods output ParameterEstimates=pe&outnum;
proc genmod data=repwork desc;
%if &byvars ne %then %do;
by &byvars;
%end;
class &clvars;
%if &offset eq  %then %do;
model &&out&outnum = &mainpred &covars /dist=&dist link=&link;
%end;
%else %do;
model &&out&outnum = &mainpred &covars /dist=&dist link=&link offset=&&off&outnum;
%end;
%if &weight ne %then %do;
  weight &weight;
%end;
run;


data pe&outnum;set pe&outnum;
Parameter = upcase(parameter);
if Parameter eq upcase("&mainpred");
Classval0 = Level1/1;
drop Level1;
run;

proc contents data=repwork  out=conmainx;
run;
data conmainx; set conmainx;
  if upcase(name) eq upcase("&mainpred") then do;
     call symputx("mainlab",Label);
     call symputx("mainfor",format);
     end;
  run;

/* Obtain main risk group counts, person time */

proc contents data=pe&outnum noprint out=conoutx;
run;
  data conoutx; set conoutx;
  if name eq 'Parameter' then call symputx("plength",Length);
  if name =: 'Classval0' then call symputx("clength",Length);
  run;

/* Create person-time variables if offset used */
%if &offset ne  %then %do;
  data trepwork; set repwork;
   ptime = exp(&&off&outnum) ;
  run;

  proc means data=trepwork noprint nway;
  class &byvars &mainpred;
  var ptime;
  output out=ptimes mean=meantime median=medtime;
  run;

  data ptimes; set ptimes;
   length Parameter $ &plength;
/*   length Classval0 $ &clength;*/
   Parameter = upcase("&mainpred");
   Classval0 = &mainpred/1;
   label meantime='Mean Time' medtime='Median Time';
   format medtime meantime 6.1;
   keep &byvars Parameter Classval0 meantime medtime;
   run;
%end;

/* Dichotomous Counts */
%if (&bout eq YES)  %then %do;
ods output crosstabfreqs=ctfc;
proc freq data=repwork;
%if &byvars ne %then %do;
by &byvars;
%end;
tables &mainpred*&&out&outnum;
run;
proc sort data=ctfc;
by &byvars &mainpred &&out&outnum;

data ct_all; set ctfc; by &byvars &mainpred &&out&outnum;
retain nsubs npcts;
if (&&out&outnum eq .) and (&mainpred ne .) then do;
  nsubs = frequency;
  npcts = put(percent,4.1);
end;
if (&&out&outnum eq 1) and (&mainpred ne .) then do;
  dsubs = frequency;
  dpcts = put(rowpercent,4.1);
end;
if last.&mainpred;
length subp $ 30;
length subd $ 30;
length EV_subs $ 30;
length Parameter $ &plength;
/*length Classval0 $ &clength;*/
subp = cat(nsubs,' (', npcts,')');
subd = cat(dsubs,' (', dpcts,')');
if &mainpred ne .;
Parameter = upcase("&mainpred");
Classval0 = &mainpred/1;
EV_subs = cat(dsubs,'/',nsubs);
label ev_subs='Events/N';
keep &byvars &mainpred subp subd nsubs npcts dsubs dpcts EV_subs Parameter Classval0 ;
run;
%end;

data pe&outnum; set pe&outnum;
if (parameter=:'Inter') or (parameter=:'Scale') then delete;
if chisq ne .;
outcome = "&&out&outnum";
Parameter = upcase(parameter);
if Parameter eq upcase("&mainpred");
run;

%if &offset ne  %then %do;
proc sort data=ptimes; by &byvars Parameter Classval0;
%end;
%if &bout eq YES %then %do;
proc sort data=ct_all; by &byvars Parameter Classval0;
%end;
proc sort data=pe&outnum; by &byvars Parameter Classval0;

%if &offset eq  and   &bout ne YES %then %do;
data pe&outnum; set pe&outnum; by &byvars Parameter Classval0;
%end;
%else %do;
data pe&outnum; merge pe&outnum 
%if &bout eq YES %then %do;
ct_all 
%end;
%if &offset ne  %then %do;
ptimes
%end;
%end; /*test*/
; by &byvars Parameter Classval0;
outcome = "&&out&outnum";
if (upcase("&mainpred")) eq upcase(parameter);
run;
%end;

/* Concatenate output from all outcomes */
data PE_all; set 
%do outnum = 1 %to &outcount;
 pe&outnum
 %end;
 ;
 length PECI $ 30;
 %if &expyn eq YES %then %do;
   Estimate = exp(Estimate);
   LowerWaldCL=exp(LowerWaldCL);
   UpperWaldCL=exp(UpperWaldCL);
 %end;
 PE = put(Estimate,best4.);
 PEU = put(UpperWaldCL,best4.);
 PEL = put(LowerWaldCL,best4.);
 if Estimate ne . then do;
 PECI = cat(PE,' (', PEL,',', PEU,')');
 end;
 else PECI = 'REF';
 drop PE PEU PEL;
 rename Estimate=PE UpperWaldCL=PEU LowerWaldCL=PEL;
 run;
proc print data=pe_all;
run;

data _null_;
length estname $ 18;
xdist = upcase("&dist");
if (xdist='BIN') then estname='OR (95% CI)';
else if (xdist='NORMAL') then estname='Beta (95% CI)';
else if (xdist='POISSON') or (xdist='NB') then estname='IDR (95% CI)';
else estname='Estimate (95% CI)';
call symputx("estname",estname);
run;

/* Combine variable names and categories if they exist */
 proc sort data=PE_all; by outcome &byvars Parameter;
 data PE_all; set PE_all; by outcome &byvars Parameter;
/* Alternating color bands by BY variable */
   retain stgrp;
   if _n_ eq 1 then stgrp = 0;
%if &byvars ne %then %do;
   if first.&&byvar&bycount then stgrp = stgrp+1;
%end;
%else %do;
   stgrp = stgrp+1;
%end;
   scolor = mod(stgrp,2);
 length betaname $ 35;
 length betaname2 $ 15;
 parmtext = put(Parameter,parmf.);
 betaname2 = put(classval0,predf.);
 betaname = cat(trim(parmtext),' ',betaname2);
 label peci="&estname" pe="&estname" outcome='Outcome' probchisq='P-value' 
 betaname='Risk Group' betaname2="&mainlab"  
%if &bout ne NO %then %do;
  EV_subs='Events/N'
  %end;
;
 format parameter parmf. outcome outf. ;
 run;

proc print data=PE_all;
run;
data &outdata; set PE_all;
length macname $ 20;
%if &outcount ne 1 %then %do;
multout = 1;
%end;
macname='fdatagmC1'; 
run;
%mend fdatagmc1;
run;

********************************************************************************;


%macro fdatagmC2(dsn,outcomes,dist,link,offset,weight,stratvars,mainpred,covars,clvars,outdata,bout=NO,expyn=YES);
/*
This macro summarizes multiple Cox regression models in a PROC REPORT format.  
It allows for multiple outcomes, one or more main predictors, multiple stratifying variables
additional covariates, and staggered entry, with a start and stop survival time.

Output from this macro can be used to create forest plots.  For situations without multiple
stratification variables, but with multiple BY variables, use %FDATAPH1.

Parameters
    dsn         Input data set name
    outcomes    Outcome variables. List with spaces.
    dist        Distribution. Must be one of "NORMAL", "BIN", "POISSON" or "NB".
    link        Link function
    offset      Offsets for Poisson/negative binomial models.  Can be left blank. Must be paired
                with the outcome variables [one offset for each outcome].  Should be the log
                value of person-time, etc.
    weight      Weight variable.  Especially for inverse probability weights. ignored if blank.
    stratvars   Stratifying variables.  The order determines the order in the figure/table.
                These variables will appear in the same column. Required!
    mainpred    A single, categorical main predictor variable, which appears in figures/tables
    covars      Covariates for adjustment.  These are not included in the figures/tables
    clvars      The CLASS statement for categorical variables.  This is the actual CLASS statement
                [except for the initial word 'class'].  See examples.
    dest        This is the ods command to open your ouput destination. The final 
                commands in the macro close this destination and open the listing destination.
                Please change this if you do not use the listing destination.
    outdata     Output data set for Reports. May include libname.
    bout        Is this a binary outcome?  Default is NO.  If this is a Poisson/NB model and has a
                dichotomous outcome, set to YES to get counts of events.
    expyn       Exponentiate estimates?  YES=yes,NO=no. Default is YES.


*/
%if (&dist eq BIN) or (&bout ne NO) %then %do;
 %let bout = YES;
%end;

/* Save current ODS destination */
data _null_; set sashelp.vdest;
call symputx("olddest",style);
run;

%let dist = %upcase(&dist);

/* make working data set */
data repwork; set &dsn;
keep &outcomes &offset &weight &stratvars &mainpred &covars;
run;

/* count outcomes, create outcome variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&outcomes,&count);
  %if &riskf ne  %then %do;
    %let out&count = &riskf; /* outcome variables */
    %let outcount = &count;
    %let count = %eval(&count+1); /* number of outcomes */
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;

/* count and create offset variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&offset,&count);
  %if &riskf ne  %then %do;
    %let off&count = &riskf; /* offset variables */
    %let count = %eval(&count+1); /* number of outcomes */
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;


/* Count STRATUM variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&stratvars,&count);
  %if &riskf ne  %then %do;
    %let stratvar&count = &riskf;
    %let stratcount = &count;
    %let count = %eval(&count+1);
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;

/* Run Genmod Models */
%do outnum = 1 %to &outcount;
  %do stratnum = 1 %to &stratcount;
  proc sort data=repwork; by &&stratvar&stratnum;
  run;
  ods output ParameterEstimates=pe&outnum._&stratnum;
  proc genmod data=repwork desc;
  by &&stratvar&stratnum;
  class &clvars;
  %if &offset eq  %then %do;
model &&out&outnum = &mainpred &covars /dist=&dist link=&link;
%end;
%else %do;
model &&out&outnum = &mainpred &covars /dist=&dist link=&link offset=&&off&outnum;
%end;
%if &weight ne %then %do;
  weight &weight;
%end;
run;
proc contents data=repwork noprint out=conmainx;
run;
data conmainx; set conmainx;
  if upcase(name) eq upcase("&mainpred") then do;
     call symputx("mainlab",Label);
     call symputx("mainfor",format);
     end;
  run;
/* Find lengths of variables,labels and formats */
proc contents data=pe&outnum._&stratnum noprint out=conoutx;
  run;
  data conoutx; set conoutx;
  if name eq 'Parameter' then call symputx("plength",Length);
 if upcase(name)=upcase("&&stratvar&stratnum") then do;
  label2 = substr(label,1,20);
  call symputx("labval",Label2);
  call symputx("forval",format);
  call symputx("slength",Length);
  end;
  run;


data pe&outnum._&stratnum;set pe&outnum._&stratnum;
length stratum1 $ &slength;
stratum1 = "&&stratvar&stratnum";
outcome = "&&out&outnum";
if (parameter=:'Inter') or (parameter=:'Scale') then delete;
if chisq ne .;
Parameter = upcase(parameter);
if Parameter eq upcase("&mainpred");
Classval0 = Level1/1;
drop Level1;
run;


/* Obtain main risk group counts, person time */


/* Create person-time variables if offset used */
%if &offset ne  %then %do;
  data trepwork; set repwork;
   ptime = exp(&&off&outnum) ;
  run;

  proc means data=trepwork noprint nway;
  class &&stratvar&stratnum &mainpred;
  var ptime;
  output out=ptimes mean=meantime median=medtime;
  run;

  data ptimes; set ptimes;
   length Parameter $ &plength;
   length stratum1 $ &slength;
   Parameter = upcase("&mainpred");
   Classval0 = &mainpred/1;
   stratum1 = "&&stratvar&stratnum";
   label meantime='Mean Time' medtime='Median Time';
   format medtime meantime 6.1;
   keep stratum1 Parameter Classval0 meantime medtime;
   run;
%end;

/* Binomial Counts */
%if &bout eq YES %then %do;
ods output crosstabfreqs=ctfc;
proc freq data=repwork;
by &&stratvar&stratnum;

tables &mainpred*&&out&outnum;
run;
proc sort data=ctfc;
by &&stratvar&stratnum &mainpred &&out&outnum;
data ct_all; set ctfc; by &&stratvar&stratnum &mainpred &&out&outnum;
retain nsubs npcts;
if (&&out&outnum eq .) and (&mainpred ne .) then do;
  nsubs = frequency;
  npcts = put(percent,4.1);
end;
if (&&out&outnum eq 1) and (&mainpred ne .) then do;
  dsubs = frequency;
  dpcts = put(rowpercent,4.1);
end;
if last.&mainpred;
length subp $ 30;
length subd $ 30;
length EV_subs $ 30;
length Parameter $ &plength;
length stratum1 $ &slength;
stratum1 = "&&stratvar&stratnum";
subp = cat(nsubs,' (', npcts,')');
subd = cat(dsubs,' (', dpcts,')');
if &mainpred ne .;
Parameter = upcase("&mainpred");
Classval0 = &mainpred/1;
EV_subs = cat(dsubs,'/',nsubs);
label ev_subs='Events/N';
keep stratum1 &mainpred subp subd nsubs npcts dsubs dpcts EV_subs Parameter Classval0 ;
run;
%end;

%if &offset ne  %then %do;
proc sort data=ptimes; by stratum1 Parameter Classval0;
%end;
%if &bout eq YES %then %do;
proc sort data=ct_all; by stratum1 Parameter Classval0;
%end;
proc sort data=pe&outnum._&stratnum; by stratum1 Parameter Classval0;

%if &offset eq  and   &bout ne YES %then %do;
data pe&outnum._&stratnum; set pe&outnum._&stratnum; /*by stratum1 Parameter Classval0;*/
%end;
%else %do;
data pe&outnum._&stratnum; merge pe&outnum._&stratnum
%if &bout eq YES %then %do;
ct_all 
%end;
%if &offset ne  %then %do;
ptimes
%end;
%end; /*test*/
;
by stratum1 Parameter Classval0;
outcome = "&&out&outnum";
if (upcase("&mainpred")) eq upcase(parameter);
run;

  proc sort data=pe&outnum._&stratnum; by stratum1;
  data pe&outnum._&stratnum; set pe&outnum._&stratnum; by stratum1;
  length strattext $ 15;
  length stratum $ 50;
  strattext = put(&&stratvar&stratnum,&forval..);
  stratum = cat("&labval",': ',strattext);
  run;
  %end;
%end;

/* Concatenate output from all outcomes */
data PE_all; set 
%do outnum = 1 %to &outcount;
  %do stratnum = 1 %to &stratcount;
     pe&outnum._&stratnum
  %end;
%end;
 ;
 length PECI $ 30;
 %if &expyn eq YES %then %do;
   Estimate = exp(Estimate);
   LowerWaldCL=exp(LowerWaldCL);
   UpperWaldCL=exp(UpperWaldCL);
 %end;
 PE = put(Estimate,best4.);
 PEU = put(UpperWaldCL,best4.);
 PEL = put(LowerWaldCL,best4.);
 if Estimate ne . then do;
 PECI = cat(PE,' (', PEL,',', PEU,')');
 end;
 else PECI ='REF';
 run;

data _null_;
length estname $ 18;
xdist = upcase("&dist");
if (xdist='BIN') then estname='OR (95% CI)';
else if (xdist='NORMAL') then estname='Beta (95% CI)';
else if (xdist='POISSON') or (xdist='NB') then estname='IDR (95% CI)';
else estname='Estimate (95% CI)';
call symputx("estname",estname);
run;

proc sort data=PE_all; by outcome stratum Parameter;;
data PE_all; set PE_all; by outcome stratum Parameter;
/* Alternating color bands by stratum */
   retain stgrp;
   if _n_ eq 1 then stgrp = 0;
   if first.stratum then stgrp = stgrp+1;
   scolor = mod(stgrp,2);
 length betaname $ 35;
 length betaname2 $ 15;
 parmtext = put(Parameter,parmf.);
 betaname2 = put(classval0,predf.);
 betaname = cat(trim(parmtext),' ',betaname2);

 label  PECI="&estname" PE="&estname" outcome='Outcome' stratum='Stratum' 
probchisq='P-value' betaname="&mainlab" betaname2="&mainlab"  
%if &bout ne NO %then %do;
  EV_subs='Events/N'
%end;
;
 format outcome outf.;
 drop stgrp;
 run;

proc print data=PE_all;
run;
data &outdata; set PE_all;
length macname $ 20;
%if &outcount ne 1 %then %do;
multout = 1;
%end;
macname='fdatagm2'; 
run;
%mend fdatagmC2;
run;

%macro repmakec(indata,byvars,dest);
/* Macro for generating reports

Parameters:

indata  Input data set. This is usually the data set specified by the OUTDATA
        parameter in the regression macros.
byvars  BY variables.  The order listed will be the order in the output.
dest    This is the ODS statement specifying where output goes.  See examples.

/* What is in the data set? */
proc contents data=&indata noprint out=repout;
run;
data repout1; set repout end=eof;
retain outyes stratyes betayes meanyes evyes peciyes pvalyes scolyes;
if _n_ eq 1 then do;
stratyes=0; outyes=0; meanyes=0; evyes=0;
end;
if upcase(name) eq 'MULTOUT' then outyes=1;
if upcase(name) eq 'STRATUM' then stratyes=1;
if upcase(name) eq 'BETANAME2' then betayes=1;
if upcase(name) eq 'MEANTIME' then meanyes=1;
if upcase(name) eq 'EV_SUBS' then evyes=1;
if upcase(name) eq 'PECI' then peciyes=1;
if upcase(name) eq 'PROBCHISQ' then pvalyes=1;
if upcase(name) eq 'SCOLOR' then scolyes=1;


if eof then do;
call symputx("outyes",outyes);
call symputx("stratyes",stratyes);
call symputx("betayes",betayes);
call symputx("meanyes",meanyes);
call symputx("evyes",evyes);
call symputx("peciyes",peciyes);
call symputx("pvalyes",pvalyes);
call symputx("scolyes",scolyes);

end;
if eof;
run;

/* Count BY variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&byvars,&count);
  %if &riskf ne  %then %do;
    %let byvar&count = &riskf;
    %let bycount = &count;
    %let count = %eval(&count+1);
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;

/* Save current ODS destination */
data _null_; set sashelp.vdest;
call symputx("olddest",style);
run;

/* Open ODS destination */
ods _all_ close;
&dest;

options missing=' ';
/* PROC REPORT version 1 */
proc report data=&indata nowd headline ls=255 ps=100 missing SPLIT='~';
title;
column 
%if &outyes eq 1 %then %do;
  outcome 
%end;
%if &betayes eq 1 %then %do;
  betaname2
%end;
%if &stratyes eq 1 %then %do;
  stratum,
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
  &&byvar&i,
%end; 
(
%if &meanyes eq 1  %then %do;  
  meantime 
%end;
%if &evyes eq 1 %then %do;
  ev_subs 
%end;
%if &peciyes eq 1 %then %do;
  peci 
%end;
%if &pvalyes eq 1 %then %do;
  probchisq
%end;
 ) ;
%if &outyes eq 1 %then %do;
  define outcome/ group order=data /*width=10*/ style(column)=[just=center];
%end;
%if &betayes eq 1 %then %do;
  define betaname2 /group order=data width=15 style(column)=[just=center];
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
  define &&byvar&i /across ' ' order=internal ;
%end; 
%if &stratyes eq 1 %then %do;
  define stratum / ' ' across order=data /*width=30*/;
%end;
%if &meanyes eq 1  %then %do;
  define meantime /group width=10 style(column)=[just=center] nozero;
%end;
%if &evyes eq 1 %then %do;
  define EV_subs /group width=15 style(column)=[just=center] nozero;
%end;
%if &peciyes eq 1 %then %do;
  define peci /group width=30 style(column)=[just=center];
%end;
%if &pvalyes eq 1 %then %do;
  define probchisq /  group width=10 style(column)=[just=center] nozero;
%end;
run;

/* PROC REPORT version 2 */
proc report data=&indata nowd headline ls=255 ps=100 missing SPLIT='~';
column %if &betayes eq 1 %then %do;
  betaname2  
%end; 
%if &outyes eq 1 %then %do;
  outcome, 
%end;
%if &stratyes eq 1 %then %do;
  stratum,
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
  &&byvar&i,
%end; 
(
%if &meanyes eq 1  %then %do;  
  meantime 
%end;
%if &evyes eq 1 %then %do;
  ev_subs 
%end;
%if &peciyes eq 1 %then %do;
  peci 
%end;
%if &pvalyes eq 1 %then %do;
  probchisq  
%end;
 ) ;
%if &outyes eq 1 %then %do;
  define outcome/ across ' ' order=data   ;
%end;
%if &betayes eq 1 %then %do;
  define betaname2 /group order=data width=10 style(column)=[just=center];
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
  define &&byvar&i /across ' ' order=internal ;
%end; 
%if &stratyes eq 1 %then %do;
  define stratum / across order=data   style(column)=[just=center];
%end;
%if &evyes eq 1 %then %do;
  define EV_subs /group style(column)=[just=center] width=8 nozero;
%end;
%if &meanyes eq 1  %then %do;  
  define meantime /group style(column)=[just=center] width=8 nozero;
%end;
%if &peciyes eq 1 %then %do;
  define peci /group width=30 style(column)=[just=center width=15%];
%end;
%if &pvalyes eq 1 %then %do;
  define probchisq /  group width=7 style(column)=[just=center] nozero;
%end;
run;

/* PROC REPORT version 3 */
proc report data=&indata nowd headline ls=255 ps=100 missing SPLIT='00'x;
title;
  column 
%if &outyes eq 1 %then %do;
  outcome 
%end;
%if &stratyes eq 1 %then %do;
  stratum
%end; 
%if &byvars ne %then %do i = 1 %to &bycount;
  &&byvar&i 
%end;
%if &betayes eq 1 %then %do;
  betaname 
%end;
, (
%if &evyes eq 1 %then %do;
  ev_subs 
%end;
%if &peciyes eq 1 %then %do;
  peci  
%end;
%if &pvalyes eq 1 %then %do;
  probchisq
%end;
 ) ;
%if &outyes eq 1 %then %do;
  define outcome/ group order=data width=20 style(column)=[just=center];
%end;
%if &betayes eq 1 %then %do;
  define betaname /across ' ' order=data width=24 
   style(column)=[just=center ];
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
  define &&byvar&i /group order=data width=10 style(column)=[just=center];
%end; 
 %if &stratyes eq 1 %then %do;
  define stratum /group order=data width=10 style(column)=[just=center];
%end;
%if &peciyes eq 1 %then %do;
  define peci /group width=20 style(column)=[just=center];
%end;
%if &evyes eq 1 %then %do;
  define ev_subs /group width=20 style(column)=[just=center] nozero;
%end;
%if &pvalyes eq 1 %then %do;
  define probchisq / group width=20 nozero
  style(column)=[just=center];
%end;
run;

/* PROC REPORT version 4 */
proc report data=&indata nowd headline ls=255 ps=130 missing SPLIT='00'x;
column 
%if &outyes eq 1 %then %do;
  outcome 
%end;
%if &stratyes eq 1 %then %do;
  stratum
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
  &&byvar&i 
%end; 
%if &betayes eq 1 %then %do;
  betaname2
%end;
%if &meanyes eq 1  %then %do;  
  meantime 
%end;
%if &evyes eq 1 %then %do;
  ev_subs 
%end;
%if &peciyes eq 1 %then %do;
  peci 
%end;
%if &pvalyes eq 1 %then %do;
  probchisq
%end;
%if &scolyes eq 1 %then %do;
  scolor
%end;
 ;
%if &outyes eq 1 %then %do;
  define outcome/ order order=data style(column)=[just=center];
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
  define &&byvar&i /order style(column)=[just=center];
%end; 
%if &stratyes eq 1 %then %do;
  define stratum /order order=data style(column)=[just=center];
%end;
%if &betayes eq 1 %then %do;
  define betaname2 /order order=data style(column)={just=center};
%end;
%if &evyes eq 1 %then %do;
  define EV_subs /order style(column)=[just=center] nozero;
%end;
%if &meanyes eq 1  %then %do;  
  define meantime /order style(column)=[just=center] nozero;
%end;
%if &peciyes eq 1 %then %do;
  define peci /order style(column)=[just=center];
%end;
%if &pvalyes eq 1 %then %do;
  define probchisq /  order nozero style(column)=[just=center] nozero;
%end;
%if &scolyes eq 1 %then %do;
  define scolor /display noprint;
%end;

%if &scolyes eq 1 %then %do;
  compute scolor;
   if scolor = 0  then do;
%if &outyes eq 1 %then %do;
   call define('outcome', "style", "style=[background=white]");
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
   call define("&&byvar&i", "style", "style=[background=white]");
%end;
 %if &stratyes eq 1 %then %do;
   call define("stratum", "style", "style=[background=white]");
%end;
%if &betayes eq 1 %then %do;
   call define('betaname2', "style", "style=[background=white]");
%end;
%if &meanyes eq 1  %then %do;  
   call define('meantime', "style", "style=[background=white]");
%end;
%if &evyes eq 1 %then %do;
   call define('EV_subs', "style", "style=[background=white]");
%end;
%if &peciyes eq 1 %then %do;
   call define('peci', "style", "style=[background=white]");
%end;
%if &pvalyes eq 1 %then %do;
   call define('probchisq', "style", "style=[background=white]");
%end;

end;
if scolor = 1  then do;
%if &outyes eq 1 %then %do;
   call define('outcome', "style", "style=[background=cxeeeeee]");
%end;
%if &byvars ne %then %do i = 1 %to &bycount;
   call define("&&byvar&i", "style", "style=[background=cxeeeeee]");
%end;
%if &stratyes eq 1 %then %do;
   call define("stratum", "style", "style=[background=cxeeeeee]");
%end;
%if &betayes eq 1 %then %do;
   call define('betaname2', "style", "style=[background=cxeeeeee]");
%end;
%if &meanyes eq 1  %then %do;  
   call define('meantime', "style", "style=[background=cxeeeeee]");
%end;
%if &evyes eq 1 %then %do;
   call define('EV_subs', "style", "style=[background=cxeeeeee]");
%end;
%if &peciyes eq 1 %then %do;
   call define('peci', "style", "style=[background=cxeeeeee]");
%end;
%if &pvalyes eq 1 %then %do;
   call define('probchisq', "style", "style=[background=cxeeeeee]");
%end;
end;
endcomp;
%end;
run;
ods _all_ close;
ods &olddest;
run;
%mend repmakec;
run;
quit;



%macro mforest(plotdata,outcount,byvars); 
/*  Macro for generating forest plots using output data sets from the regression macros.

Parameters

    plotdata    Input data set. This is usually the data set specified by the OUTDATA
                parameter in the regression macros.
    outcount    Number of outcomes.  Determines whether the outcome column is created. 
    byvars      If there are BY variables, list them.  The order listed is the order
                in the output.

Output data sets:
    fplotit1        Used for version 1 plot
    fplotit2        Used for version 2 & 3 plots
    fplotit3        Used for version 4 plot
*/

/* What is in the data set? */
proc contents data=&plotdata noprint out=repout;
run;
data repout1; 
length label $ 15;
set repout end=eof;
retain stratyes storyes outyes ooryes betayes cv0yes boryes meanyes evyes refyes;
if _n_ eq 1 then do;
stratyes=0; storyes=0; outyes=0; ooryes=0; betayes=0; cv0yes = 0;
 boryes=0; meanyes=0; evyes=0;refyes=0;
end;
if upcase(name) eq 'PE' then call symput("pecilab",label);
if upcase(name) eq 'STRATUM' then stratyes=1;
if upcase(name) eq 'STORDER' then storyes=1;
if upcase(name) eq 'MULTOUT' then outyes=1;
if upcase(name) eq 'OORDER' then ooryes=1;
if upcase(name) eq 'BETANAME2' then betayes=1;
if upcase(name) eq 'CLASSVAL0' then cv0yes=1;
if upcase(name) eq 'BORDER' then boryes=1;
if upcase(name) eq 'MEANTIME' then meanyes=1;
if upcase(name) eq 'EV_SUBS' then evyes=1;
if upcase(name) eq 'FSHADE' then refyes=1;

if eof then do;
call symputx("stratyes",stratyes);
call symputx("outyes",outyes);
call symputx("meanyes",meanyes);
call symputx("evyes",evyes);
call symputx("betayes",betayes);
call symputx("refyes",refyes);
if storyes eq 1 then call symputx("storder","storder");
   else call symputx("storder","stratum");
if ooryes eq 1 then call symputx("oorder","oorder");
   else call symputx("oorder","outcome");

if boryes eq 1 then call symputx("border","border");
   else if cv0yes eq 1 then call symputx("border","classval0");
   else call symputx("border",betaname2);
end;
if eof;
run;



/* Count BY variables */
%let count = 1;
%let done = 0;
%do %while (&done ne 1);
  %let riskf = %qscan(&byvars,&count);
  %if &riskf ne  %then %do;
    %let byvar&count = &riskf;
    %let bycount = &count;
    %let count = %eval(&count+1);
  %end;
  %else %do;
    %let done = 1;
    %end;
%end;

%if &byvars ne  %then %do;
  proc contents data=&plotdata noprint out=conoutx;
  run;
  data conoutx; set conoutx;
  %do i = 1 %to &bycount;
  if upcase(name)=upcase("&&byvar&i") then do;;
  call symputx("forval&i",format);
  output;
  end;
  %end;
%end;
proc sort data=&plotdata; by &oorder
%if &stratyes eq 1 %then %do;
  &storder 
%end;
&byvars &border;
run;
data tmpforest; 
length PECI $ 30;
set &plotdata;by &oorder
%if &stratyes eq 1 %then %do;
  &storder
%end;
&byvars &border;
where PE ne .;
keep outcome sorder &byvars PE PEu PEl betaname  
indent id ctext PEci &border 
%if &stratyes eq 1 %then %do;
  stratum &storder
%end;
%if &refyes eq 1 %then %do;
  fshade
%end;
;
retain flaghr;
if _n_ eq 1 then flaghr=0;
length ctext $ 20;
sorder = _n_;
id=3;
indent = 3;
ctext=betaname;
output;
%if &outcount ne 1 %then %do;
    if first.&oorder then do;
       PEci=' ';PE=.; PEL=.;PEU=.;id=1;indent=id-1;ctext=outcome;
       output;
    end;
%end;

%if &stratyes eq 1 %then %do;
  if first.&storder then do;
     if flaghr eq 0 then do; PEci="&pecilab"; flaghr=1;end;
     else do; PECI = ' '; 
     end;
  PE=.; PEL=.;PEU=.;id=2;indent=id-1;ctext=stratum;
  output;
  end;
%end;
%if &byvars ne  %then %do bcount = 1 %to &bycount;
if first.&&byvar&bcount then do;
  if flaghr eq 0 then do; PEci="&pecilab"; flaghr=1;end;
  else do; PECI = ' '; end;
  PE=.; PEL=.;PEU=.;id=2;indent=0+&bcount+&stratyes;
  %if &&forval&bcount ne %then %do;
    ctext=put(&&byvar&bcount,&&forval&bcount...);
  %end;
  %else %do;
    ctext = &&byvar&bcount;
  %end;
output;
end;
%end;
run;
data tmpforest; set tmpforest;
if ctext ne ' ';
outcount = &outcount-1;
stratcount = &stratyes;
%if &byvars ne  %then %do;
  bycount = &bycount;
%end;
%else %do;
  bycount = 0;
%end;
if id eq 3 then indent = outcount+stratcount+bycount; 
run;
proc sort data=tmpforest; by sorder &oorder 
%if &stratyes eq 1 %then %do;
  &storder
%end;
&byvars &border id;
run;

data fplotit1; set tmpforest; 

sorder = _n_;
%if &byvars ne %then %do;
  if id eq 8 then do;
    indent = (&outcount ne 1) + bycount;
    id = indent+1;
  end;
%end;
%else %do;
  if id eq 8 then indent = (&outcount ne 1);
%end;
%if (&byvars eq  ) and (&storder eq ) %then %do;
 if _n_ eq 1 then PECI="&pecilab";
%end;
label PE = "&pecilab";
run;
proc sort data=fplotit1; by sorder &oorder &byvars &border id;
run;
/* Graphics section.  This section can be copied elsewhere,modified and executed as
   there is only one macro variable in it:  The input data set.  */
data attrmap;
  length id textcolor textweight $10;
  id='text'; value='1'; textcolor='Black'; textweight='bold'; output;
  id='text'; value='2'; textcolor='Black'; textweight='normal'; output;
  id='textColor'; value='1'; textcolor='Red'; textweight='bold'; output;
  id='textColor'; value='2'; textcolor='Black'; textweight='bold'; output;
run;

/* Version 1: Simple Forest Plot linear scale*/
proc sgplot data=fplotit1 noborder dattrmap=attrmap;
title;
  scatter    y=sorder x=PE / xerrorlower=PEl xerrorupper=PEu
             markerattrs=(symbol=circlefilled size=10) noerrorcaps ERRORBARATTRS=(thickness=2 color=red);
  yaxistable ctext / position=left location=inside textgroup=id textgroupid=textcolor
             indent=40 indentweight=indent labelhalign=left nolabel VALUEATTRS=(Size=10);
  yaxis      display=none reverse fitpolicy=none ;
  xaxis      label="&pecilab" valueattrs=(size=12) labelattrs=(size=13) /*type=log  values=(0.5, 1 to 8 by 1)*/;
  refline 1 /axis=x lineattrs=(thickness=2 color=black);
run;
quit;

/* Version 2: Forest Plot with shading, log scale */
data fplotit2; set fplotit1;
  if PE ne . then ref = sorder; ** identify lines to shade **;
run;

proc sgplot data=fplotit2 noborder dattrmap=attrmap;
title;
scatter    y=sorder x=PE / xerrorlower=PEL xerrorupper=PEU
           markerattrs=(symbol=circlefilled size=10) noerrorcaps ERRORBARATTRS=(thickness=2 color=red);
yaxistable ctext / position=left location=inside textgroup=id textgroupid=textcolor
           indent=30 indentweight=indent labelhalign=left nolabel VALUEATTRS=(Size=10);
yaxis      reverse display=none type=discrete colorbands=odd COLORBANDSATTRS=( transparency=1); ;
xaxis      label="&pecilab" labelattrs=(size=13) valueattrs=(size=12) type=log  ;
refline    ref / lineattrs=(color=cx999999) discretethickness=1 transparency=0.8;
refline    1 /axis=x lineattrs=(thickness=2 color=black);
run;
quit;

/* Version 3: Forest Plot with shading, log scale and PECI table */

proc sgplot data=fplotit2 noborder dattrmap=attrmap;
title;
scatter    y=sorder x=PE / xerrorlower=PEL xerrorupper=PEU
           markerattrs=(symbol=circlefilled size=10) noerrorcaps ERRORBARATTRS=(thickness=2 color=red);
yaxistable ctext / position=left location=inside textgroup=id textgroupid=textcolor
           indent=30 indentweight=indent labelhalign=left nolabel VALUEATTRS=(Size=10);
yaxistable PECI / position=right location=inside textgroup=id textgroupid=textcolor
           valuejustify=center nolabel VALUEATTRS=(Size=10);
yaxis      reverse display=none type=discrete colorbands=odd colorbandsattrs=( transparency=1); ;
xaxis      label="&pecilab" labelattrs=(size=13) valueattrs=(size=12) type=log ;
refline    ref / lineattrs=(color=cx999999) discretethickness=1 transparency=0.8;
refline    1 /axis=x lineattrs=(thickness=2 color=black);
run;
quit;

/* Version 4: Compact plot version -- this has more macro variables!! */
data fplotit3;set fplotit1;
  if pe ne .;
  run;
proc sort data=fplotit3; by outcome &byvars 
%if &stratyes eq 1 %then %do;
 stratum
%end;
&border;
run;
data fplotit3; set fplotit3;by  outcome &byvars 
%if &stratyes eq 1 %then %do;
 stratum
%end;
&border;
if first.outcome eq 0 then outcome=' ';
%if &byvars ne %then %do i = 1 %to &bycount;
 if first.&&byvar&i  eq 0 then &&byvar&i =' ';
%end; 
%if &stratyes eq 1 %then %do;
if first.stratum eq 0 then stratum=' ';
%end;
sorder = _n_;
%if &refyes ne 1 %then %do;
if mod(sorder,4) in(1,2) then ref=sorder;  *  for shading in pairs of line;
%end;
%else %do;
if fshade eq 1 then ref = sorder;
%end;
run;
proc sgplot data=fplotit3 noborder dattrmap=attrmap;
title;
scatter      y=sorder x=PE / xerrorlower=PEL xerrorupper=PEU
             markerattrs=(symbol=circlefilled size=10) noerrorcaps ERRORBARATTRS=(thickness=2 color=red);
yaxistable   outcome  / position=left location=inside 
             valuejustify=center nolabel VALUEATTRS=(Size=12 weight=bold color=red);
%if &byvars ne %then %do i = 1 %to &bycount;
yaxistable   &&byvar&i    / position=left location=inside 
             valuejustify=center nolabel VALUEATTRS=(Size=10 weight=bold color=black);
%end; 
%if &stratyes eq 1 %then %do;
yaxistable   stratum / position=left location=inside 
             valuejustify=center nolabel VALUEATTRS=(Size=10 weight=bold color=black);
%end;
yaxistable   betaname  / position=left location=inside 
             valuejustify=center nolabel VALUEATTRS=(Size=10 weight=normal color=black);
yaxis        reverse display=none type=discrete colorbands=odd colorbandsattrs=( transparency=1); ;
xaxis        label="&pecilab" labelattrs=(size=13) valueattrs=(size=12) type=log ;
refline      ref / lineattrs=(color=cx999999) discretethickness=1 transparency=0.8;
refline      1 /axis=x lineattrs=(thickness=2 color=black);
run;
quit;
%mend mforest;
run;


%macro fmergitr(template,shuffle,outrep,multout=1,);
/*  This macro allows for rearranging data sets before generating reports or
    forest plots, or it can be used to merge output from multiple regression
    macro calls.  See the companion document for further explanantion.

Parameters:

template    Data set with information on the input data sets
shuffle     Data set with instructions on variable order, renaming, formats
            and labels
outrep      Output data set -- to be used by reporting and forest plot macros
multout     Are there multiple outcomes in the output of the regression macros?
            Default is '1' [1=Yes, there are multiple outcomes].

/* Extract info from template data set */
data _null_; set &template end=eof;
rname = cat("rname",_n_);
fname = cat("fname",_n_);
glab = cat("glab",_n_);
gord = cat("gord",_n_);
call symputx(rname,rdsn);
call symputx(fname,fdsn);
call symputx(glab,grplab);
call symputx(gord,grporder);
if eof then call symputx("numruns",_n_);
run;

/* Modify individual data files */
%do dset = 1 %to &numruns;
data file&dset; set &&rname&dset;
length newvar $ 20;
newvar = "&&glab&dset";
norder=&dset;
/*drop Parameter;*/
run;
%end;

/* Combine All Report Data */
data outrep; set 
%do dset = 1 %to &numruns;
file&dset
%end;
;
by norder;
run;
proc print data=outrep;
run;

/* Are standard variables in the data set? */
proc contents data=outrep out=contall;
run;
data contall; set contall end=eof;
retain outyes stratyes betayes meanyes evyes peciyes pvalyes peyes pelyes peuyes classyes moutyes;
if _n_ eq 1 then do;
outyes=0; stratyes=0; betayes=0; meanyes=0; evyes=0;
peciyes=0; pvalyes=0; peyes=0; pelyes=0; peuyes=0; classyes=0; moutyes=0;
end;
if upcase(name) eq 'OUTCOME' then outyes=1;
if upcase(name) eq 'STRATUM' then stratyes=1;
if upcase(name) eq 'BETANAME2' then betayes=1;
if upcase(name) eq 'MEANTIME' then meanyes=1;
if upcase(name) eq 'EV_SUBS' then evyes=1;
if upcase(name) eq 'PECI' then peciyes=1;
if upcase(name) eq 'PROBCHISQ' then pvalyes=1;
if upcase(name) eq 'PE' then peyes=1;
if upcase(name) eq 'PEL' then pelyes=1;
if upcase(name) eq 'PEU' then peuyes=1;
if upcase(name) eq 'CLASSVAL0' then classyes=1;
if upcase(name) eq 'MULTOUT' then moutyes=1;

if eof then do;
call symputx("yes1",outyes);
call symputx("yes2",stratyes);
call symputx("yes3",betayes);
call symputx("yes4",meanyes);
call symputx("yes5",evyes);
call symputx("yes6",peciyes);
call symputx("yes7",pvalyes);
call symputx("yes8",peyes);
call symputx("yes9",pelyes);
call symputx("yes10",peuyes);
call symputx("yes11",classyes);
call symputx("moutyes",moutyes);

end;
if eof;
run;

data _null_;
call symputx("varc1","outcome");
call symputx("varc2","stratum");
call symputx("varc3","betaname2");
call symputx("varc4","meantime");
call symputx("varc5","EV_subs");
call symputx("varc6","PECI");
call symputx("varc7","probchisq");
call symputx("varc8","PE");
call symputx("varc9","PEL");
call symputx("varc10","PEU");
call symputx("varc11","classval0");
run;

/* Extract data from shuffle data set */
data _null_; set &shuffle end=eof;
Oname = cat("oname",_n_);
nname = cat("nname",_n_);
nlab = cat("nlab",_n_);
nfmt = cat("nfmt",_n_);
nyes = cat("nyes",_n_);
lyes = cat("lyes",_n_);
if newlabel eq ' ' then labyes = 0;
  else labyes = 1;

if nname eq ' ' then nnamey=0;
  else nnamey=1;
call symputx(Oname,oldstuff);
call symputx(nname,newstuff);
call symputx(nlab,newlabel);
call symputx(lyes,labyes);
call symputx(nfmt,newFormat);
call symputx(nyes,nnamey);
/*if (_n_ eq 3) and upcase(newstuff) ne "betaname2" then newbeta=1;*/
/*  else newbeta=0;*/
/*call symputx("nbeta2","newbeta");*/
run;

data intest;
length nname Oname nlab fmt $ 20;
%do jj = 1 %to 11;
 loop = &jj;
 nname = "&&nname&jj";
 Oname = "&&oname&jj";
 nlab = "&&nlab&jj";
 fmt = "&&nfmt&jj";
 yesvar = "&&yes&jj";
 output;
 %end;
 run;
 proc print data=intest;
 run;

data &outrep; set outrep; by norder;
/* If the variable exists in the input data set, assign to new xzvar list */
%do kr = 1 %to 11;
  %if &&nyes&kr ne %then %do;
    xzvar&kr = &&nname&kr ;
  %end;
%end;
rename 
  %do dset = 1 %to 11;
    %if &&nyes&dset ne %then %do;
       xzvar&dset = &&Oname&dset
    %end;
  %end;
;
  drop
  %do dset = 1 %to 11;
    %if &&yes&dset eq 1 %then %do;
      &&Oname&dset
    %end;
  %end;
;
  %do dset = 1 %to 8;
    %if &&lyes&dset eq 1   %then %do;
      label xzvar&dset = "&&nlab&dset";
    %end;
  %end;

  %do dset = 1 %to 8;
    %if &&nfmt&dset ne %then %do;
      format xzvar&dset &&nfmt&dset;
    %end;
  %end;

%if &multout eq 1 %then %do;
  multout = 1;
%end;
run;
data &outrep; set &outrep; by norder;
betaname=betaname2;
run;

%mend fmergitr;
run;

